"""Logging interface to enforce all logger wrappers to follow the same format"""
from typing import Any, Dict, Optional, Union
import abc
import pandas as pd
from pandas.core.api import DataFrame
from es.elastic.api import connect
from es.baseapi import BaseConnection

from ..constants.config import BasicConfig, BoolConfig, StringConfig
from ..constants.keys import LoggerKeys
from ..helpers.formats import format_elk_url


class LoggerInterface(abc.ABC):
    """
    The loggers interface used for all logger types.
    """

    def __init__(self, **kwargs) -> None:
        # Required keys
        self.appname: str = kwargs[LoggerKeys.APPNAME.value]

        # Optional keys
        self.debug: bool = (
            True
            if kwargs.get(LoggerKeys.DEBUG.value) is None
            else kwargs.get(LoggerKeys.DEBUG.value)
        )
        self.version: str = (
            kwargs.get(LoggerKeys.VERSION.value) or StringConfig.VERSION.value
        )

        # elasticsearch keys
        self.config: Dict[str, Any] = kwargs
        if not self.debug:
            self.host: str = kwargs[LoggerKeys.HOST.value]
            self.index: str = kwargs[LoggerKeys.INDEX.value]
            self.username: str = kwargs[LoggerKeys.USERNAME.value]
            self.pw: Any = kwargs[LoggerKeys.PW.value]

    @abc.abstractmethod
    def log(
        self,
        level: str,
        logdata: Union[str, Dict[str, Any]],
        custom_func_name: Optional[str] = None,
        use_custom_logdata: Optional[bool] = None,
    ) -> None:
        """
        Log data to the appropriate loggers.
        """

    @abc.abstractmethod
    async def async_log(
        self,
        level: str,
        logdata: Union[str, Dict[str, Any]],
        custom_func_name: Optional[str] = None,
        use_custom_logdata: Optional[bool] = None,
    ) -> None:
        """
        Log data to the appropriate loggers.
        """

    @abc.abstractmethod
    def query(
        self, custompayload: Optional[Dict[str, Any]], size: int, cache: bool
    ) -> Any:
        """
        Make query to elasticsearch.
        """

    @abc.abstractmethod
    async def async_query(
        self, custompayload: Optional[Dict[str, Any]], size: int, cache: bool
    ) -> Any:
        """
        Make query to elasticsearch.
        """

    def sql_query(self, query: str, **kwargs) -> DataFrame:
        """
        Make query to elasticsearch-dbapi
        """
        if self.debug:
            return None

        conn: BaseConnection = connect(
            host=format_elk_url(self.config, BoolConfig.USE_ES_DB.value),
            port=self.config.get(LoggerKeys.PORT.value) or BasicConfig.PORT.value,
            scheme=StringConfig.SCHEME.value,
            user=self.username,
            password=self.pw,
            verify_certs=BoolConfig.VERIFY_CERTS.value,
        )
        data: DataFrame = pd.read_sql(query, kwargs.get("conn"))
        conn.close()
        return data
